==> openmsx_0.5.1.orig.tar.gz: openmsx-0.5.1/src/HostCPU.cc <==
// $Id: HostCPU.cc,v 1.6 2004/12/15 09:09:42 m9710797 Exp $

#include "HostCPU.hh"
#include "openmsx.hh"
#include <cassert>
#include <cstdio>
#include <iostream>
#include <iomanip>

namespace openmsx {

HostCPU::HostCPU()
{
	PRT_DEBUG("probing host CPU...");

	mmxFlag = false;
	mmxExtFlag = false;
	#ifdef ASM_X86 
		// Is CPUID instruction supported?
		unsigned hasCPUID;
		asm (
			// Load EFLAGS into EAX.
			"pushfl;"
			"popl	%%eax;"
			// Save current value.
			"movl	%%eax,%%ebx;"
			// Toggle bit 21.
			"xorl	$0x200000, %%eax;"
			// Load EAX into EFLAGS.
			"pushl	%%eax;"
			"popfl;"
			// Load EFLAGS into EAX.
			"pushfl;"
			"popl	%%eax;"
			// Did bit 21 change?
			"xor	%%ebx, %%eax;"
			"andl	$0x200000, %%eax;"
			: "=a" (hasCPUID) // 0
			: // no input
			: "ebx"
			);
		if (hasCPUID) {
			// Which CPUID calls are supported?
			unsigned highest;
			asm (
				"cpuid;"
				: "=a" (highest) // 0
				: "0" (0) // 1: function
				: "ebx", "ecx", "edx"
				);
			if (highest >= 1) {
				// Get features flags.
				unsigned features;
				asm (
					"cpuid;"
					: "=d" (features) // 0
					: "a" (1) // 1: function
					: "ebx", "ecx"
					);
				PRT_DEBUG("CPU flags: " << std::hex << std::setw(8) << std::setfill('0') << std::uppercase << features);
				mmxFlag = features & 0x800000;
				bool sseFlag = features & 0x2000000;
				mmxExtFlag = mmxFlag && sseFlag;
			}
		}
	#endif

	PRT_DEBUG("MMX:              " << mmxFlag);
	PRT_DEBUG("MMX extensions:   " << mmxExtFlag);
}

} // namespace openmsx

/*
int main(char** argv, int argc) {
	openmsx::HostCPU::getInstance();
	return 0;
}
*/

==> openmsx_0.6.2.orig.tar.gz: openmsx-0.6.2/src/HostCPU.cc <==
// $Id: HostCPU.cc 5789 2006-10-18 17:13:07Z andete $

#include "HostCPU.hh"
#include "openmsx.hh"

namespace openmsx {

HostCPU::HostCPU()
{
	mmxFlag = false;
	mmxExtFlag = false;
	#ifdef ASM_X86_32
		// Note: On Mac OS X, EBX is in use by the OS, so we have to restore it.
		// Is CPUID instruction supported?
		unsigned hasCPUID;
		asm (
			// Load EFLAGS into EAX.
			"pushfl;"
			"popl	%%eax;"
			// Save current value.
			"movl	%%eax,%%ecx;"
			// Toggle bit 21.
			"xorl	$0x200000, %%eax;"
			// Load EAX into EFLAGS.
			"pushl	%%eax;"
			"popfl;"
			// Load EFLAGS into EAX.
			"pushfl;"
			"popl	%%eax;"
			// Did bit 21 change?
			"xor	%%ecx, %%eax;"
			"andl	$0x200000, %%eax;"
			: "=a" (hasCPUID) // 0
			: // no input
			: "ecx"
			);
		if (hasCPUID) {
			// Which CPUID calls are supported?
			unsigned highest;
			asm (
				"pushl	%%ebx;"
				"cpuid;"
				"popl	%%ebx;"
				: "=a" (highest) // 0
				: "0" (0) // 1: function
				: "ecx", "edx"
				);
			if (highest >= 1) {
				// Get features flags.
				unsigned features;
				asm (
					"pushl	%%ebx;"
					"cpuid;"
					"popl	%%ebx;"
					: "=d" (features) // 0
					: "a" (1) // 1: function
					: "ecx"
					);
				mmxFlag = features & 0x800000;
				bool sseFlag = features & 0x2000000;
				mmxExtFlag = mmxFlag && sseFlag;
			}
		}
	#endif
	#ifdef ASM_X86_64
		// X86_64 machines always have mmx and mmxExt
		mmxFlag = true;
		mmxExtFlag = true;
	#endif

	PRT_DEBUG("MMX:              " << mmxFlag);
	PRT_DEBUG("MMX extensions:   " << mmxExtFlag);
}

} // namespace openmsx
